# -*- coding: utf-8 -*-
# pylint: disable=R1705, R0205

# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# --------------------------------------------------------------------------
"""File for operating on an AD agent Backupset.

adbackupset is the only class defined in this file.

Function:
    azuread_browse_double_query     create browse options for objects in folder

    azuread_browse_double_query_adv    create browse options for objects attrbute

    azuread_browse_options_builder    build browse options for azure ad browse
Class:

    AzureADBackupset:  Derived class from Backuset Base class, representing a
                            Azure AD agent backupset, and to perform operations on that backupset

        _azuread_browse_basic : Do basic browse with option

        _azuread_browse_meta  : Get Azure ad folder meta information

        _azuread_browse_folder : Get Azure objects based on the folder type

        _adv_attributes : Get Azure AD object attribute

        browse() : Overwrite default browse operation

        _process_browse_repsonse : process the browse result

        azuread_get_metadata : create azure ad object meta data information
        
        __prepare_search_json : Prepare search json for search api request

        get_search_response : Get search response from search api

        view_attributes_url_builder : Build view attribute url

        get_view_attribute_response : Get view attribute response from view attribute url

"""

from __future__ import unicode_literals
import base64
from ..backupset import Backupset
from ..exception import SDKException


class AzureAdBackupset(Backupset):
    """ Azure AD agent backupset class """

    def _azuread_browse_basic(self, options):
        """ do basic browse activity with options
            Args:
                options    (dict)    browse option from impoort
                                adv_attributes    get object advanced attribute
            Return:
                count     (int)    return count from browse
                result    (list)    objects list from browse
            Raise:
                101    If browse return Nothing
        """
        options = self._prepare_browse_options(options)

        request_json = self._prepare_browse_json(options)
        request_json = self.azuread_browse_double_query(options, request_json)
        flag, response = self._cvpysdk_object.make_request("POST",
                                                           self._BROWSE,
                                                           request_json)
        if flag:
            count, result = self._process_browse_response(flag, response, options)
        else:
            raise SDKException('Response', '101', self._update_response_(response.text))
        return count, result

    def _azuread_browse_meta(self, options):
        """ get basic browse related meta data
            Args:
                options    (dict)    browse option from impoort
            Return:
                azure_meta    (dict)    azure ad meta data from browse result
            Raise:
                None
        """
        if "meta" in options:
            azure_meta = options['meta']
        else:
            azure_meta = {}
        count, result = self._azuread_browse_basic(options)
        if len(result) == 1 and result[0]['objType'] == 1:
            newid = result[0]['commonData']['id']
            options['filters'] = [(_[0], newid, _[2]) for _ in options['filters']]
            name, metainfo = self.azuread_browse_obj_meta(result[0])
            azure_meta['root'] = metainfo
            options['meta'] = azure_meta
            self._azuread_browse_meta(options)
        else:
            for r_ in result:
                if r_['objType'] == 100:
                    options['filters'] = [("76", r_['commonData']['id'], "9"),
                                        ("125", "FOLDER")]
                    count_, metainfo_ = self._azuread_browse_basic(options)
                    for i_ in metainfo_:
                        name, metainfo = self.azuread_browse_obj_meta(i_)
                        azure_meta[name] = metainfo
                else:
                    name, metainfo = self.azuread_browse_obj_meta(r_)
                    azure_meta[name] = metainfo
        return azure_meta

    def _azuread_browse_folder(self, options):
        """ browse folder content
            Args:
                options    (dict)    browse option from impoort
            Return:
                count     (int)    return count from browse
                result    (list)    objects list from browse
            Raise:
                None
        """
        azure_meta_mapper = {
            "user" : { "displayname" : "Users", "browsetype" : 2, "browsestring" : "USER"},
            "group" : { "displayname" : "Groups", "browsetype" : 3, "browsestring": "GROUP"},
            "reg_app" : {"displayname" : "App registrations", "browsetype" : 5, "browsestring" : "APPLICATION"},
            "ent_app" : { "displayname" : "Enterprise applications","browsetype": 6, "browsestring" : "SERVICE_PRINCIPAL"},
            "ca_policy" : { "displayname" : "Policies","browsetype": 11, "browsestring" : "CONDITIONAL_ACCESS_POLICY"},
            "ca_name_location" : { "displayname" : "Named locations","browsetype": 12, "browsestring" : "NAMED_LOCATION"} ,
            "ca_auth_context" : { "displayname" : "Authentication context","browsetype": 13, "browsestring" : "AUTHENTICATION_CONTEXT"},
            "ca_auth_strength" : { "displayname" : "Authentication strengths","browsetype": 14, "browsestring" : "AUTHENTICATION_STRENGTH"},
            "role" : { "displayname" : "Roles","browsetype": 15, "browsestring" : "DIRECTORY_ROLE_DEFINITIONS"},
            "admin_unit" : { "displayname" : "Admin units","browsetype": 16, "browsestring" : "ADMINISTRATIVE_UNIT"}}
        azure_meta = options['meta']
        newid = azure_meta[azure_meta_mapper[options['folder']]['displayname']]['id']
        options['filters'] = [("76", newid, "9"),
                              ("125", azure_meta_mapper[options['folder']]['browsestring'])]
        if "search" in options:
            if isinstance(options['search'], dict):
                search_dict = options['search']
                if "obj_id" in search_dict:
                    options['filters'].append(("130", search_dict['obj_id']))
                if "source" in search_dict:
                    if search_dict['source'] == "AzureAD":
                        options['filters'].append(("128", "0"))
                    elif search_dict['source'] == "WinAD":
                        options['filters'].append(("128", "1"))
            else:
                options['filters'].append(("30", options['search']))

        del(options['folder'])
        del(options['meta'])
        del(options['path'])
        count, results = self._azuread_browse_basic(options)
        results = self._process_result_format(results)
        return count, results

    def browse(self, *args, **kwargs):
        """Browses the content of the Backupset.
            Args:
                args    list    args passed for browse
                kwargs    dict    dict passed for browse
            Return:
                count     (int)    return count from browse
                browse_result    (list)    objects list from browse
            Raise:
                None
        """
        if args and isinstance(args[0], dict):
            options = args[0]
        else:
            options = kwargs
        options = self.azuread_browse_options_builder(options)
        azure_meta = self._azuread_browse_meta(options)
        options['meta'] = azure_meta
        count, browse_result = self._azuread_browse_folder(options)

        return count, browse_result

    def _process_browse_response(self, flag, response, options):
        """Retrieves the items from browse response.
            Args:
                flag    (bool)  --  boolean, whether the response was success or not
                response (dict)  --  JSON response received for the request from the Server
                options  (dict)  --  The browse options dictionary
            Returns:
                list - List of only the file / folder paths from the browse response
                dict - Dictionary of all the paths with additional metadata retrieved from browse
            Raises:
                SDKException:
                    if failed to browse/search for content
                    if response is empty
                    if response is not success
        """
        metadatas = []
        if flag:
            if response.json():
                response_json = response.json()
                if response_json and 'browseResponses' in response_json:
                    _browse_responses = response_json['browseResponses']
                    for browse_response in _browse_responses:
                        if "browseResult" in browse_response:
                            browse_result = browse_response['browseResult']
                            browseresulttcount = browse_result['totalItemsFound']
                            if 'dataResultSet' in browse_result:
                                result_set = browse_result['dataResultSet']
                    if not result_set and result_set != []:
                        raise SDKException('Backupset', '102',
                                           "Failed to browse for subclient backup content")
                    else:
                        for result in result_set:
                            metadata = self.azuread_get_metadata(result)
                            metadatas.append(metadata['azureADDataV2'])
            else:
                raise SDKException("Backupset", "102", "response is not valid")
        else:
            raise SDKException('Response', '101', self._update_response_(response.text))
        return browseresulttcount, metadatas

    def _process_result_format(self, results):
        """
        process the browse result to original data format
        Args:
            results     (list)  search results list
        return
            results     (list)  search results list
        """
        for _ in results:
            _['azureid'] =  _['commonData']['id'].replace("x","-")
        return results
    def azuread_get_metadata(self, result):
        """ Get azure ad meta data for browse result
            Args:
                result    (list)    objects list from browse
            Return:
                metadata    (dict)    azure ad browse meta data
            Raise:
                110    can't find meta data
        """
        metadata = {}
        if "advancedData" in result:
            if "browseMetaData" in result['advancedData']:
                metadata = result['advancedData']['browseMetaData']
                if "azureADDataV2" in metadata:
                    metadata['azureADDataV2']['guid'] = result['advancedData']['objectGuid']
                else:
                    raise SDKException('Backupset', '110',
                                       "Azure AD meta data is not found")
        return metadata

    def azuread_browse_obj_meta(self, obj_):
        """ get azure ad obj meta info
            Args:
                obj_    (obj)    azuare ad object
            Return:
                name    (str)    azure ad display name
                metainfo     (dict)    azure ad browse meta data
            Raise:
                None
        """
        name = obj_['commonData']['displayName']
        metainfo = {}
        metainfo['id'] = obj_['commonData']['id']
        metainfo['azureid'] = metainfo['id'].replace("x", "-")
        metainfo['name'] = name
        metainfo['guid'] = obj_['guid']
        metainfo['type'] = obj_['objType']
        return name, metainfo

    def azuread_browse_double_query(self, options, request_json):
        """ create request json for azure ad based on double query
            Args:
                options    (dict)    browse option from impoort
                request_json    (json)    request json file from basic request class
            Return:
                request_json    (json)    request json with addittional options
            Raise:
                None
        """
        request_json['queries'] = [{
                        "type": "0",
                        "queryId": "0",
                        "whereClause" :[],
                        "dataParam": {
                            "sortParam": {
                                "ascending": True,
                                "sortBy": [126]
                            },
                            "paging": {
                                "pageSize": int(options['page_size']),
                                "skipNode": int(options['skip_node']),
                                "firstNode": 0
                            }
                        }
                    },
                    {   "type": "1",
                        "queryId": "1",
                        "whereClause": [],
                        "aggrParam"  : {'aggrType': 4,
                                        'field': 0}}]

        if options['filters']:
            for filter_ in options['filters']:
                filter_dict = {
                    'connector': 0,
                    'criteria': {
                        'field': filter_[0],
                        'values': [filter_[1]]}}
                if len(filter_) == 3:
                    filter_dict['criteria']['dataOperator'] = int(filter_[2])
                if filter_[0] == "125":
                    del (filter_dict['connector'])
                request_json['queries'][0]['whereClause'].append(filter_dict)
                request_json['queries'][1]['whereClause'].append(filter_dict)

        del(request_json['paths'])
        return request_json

    def azuread_browse_options_builder(self, options):
        """ build browse options
            Args:
                options    (dict)    browse option from impoort
            Return:
                options    (list)    create formated options based on import
            Raise:
                None
        """
        if "filters" not in options:
            options['filters'] = [("76", "00000000000000000000000000000001", "9"),
                                  ("76", "00000000000000000000000000000001", "9")]
        if "operation" not in options:
            options['operation'] = "browse"
            options['page_size'] = 20
            options['skip_node'] = 0
        return options

    def __prepare_search_json(self, options):
        """
        performs view_properties (Cvpysdk Api call)
        Args:
            options     (dict)      example {"to_time":(epoch) ,
                                            "subclient_id":(string/optional),
                                            "attribute": "attribute to perform search "}
            Return:     (dict)      view properties
        """
        options["subclient_id"] = self.subclients.all_subclients['default']['id']

        request_json = {
            "mode": 4,
            "advSearchGrp": {
                "commonFilter": [
                    {
                        "filter": {
                            "interFilterOP": 2,
                            "filters": [
                                {
                                    "field": "CISTATE",
                                    "intraFieldOp": 0,
                                    "fieldValues": {
                                        "values": [
                                            "1"
                                        ]
                                    }
                                },
                                {
                                    "field": "IS_VISIBLE",
                                    "intraFieldOp": 0,
                                    "fieldValues": {
                                        "values": [
                                            "true"
                                        ]
                                    }
                                }
                            ]
                        }
                    }
                ],
                "fileFilter": [
                    {
                        "interGroupOP": 2,
                        "filter": {
                            "interFilterOP": 2,
                            "filters": [
                                {
                                    "field": "BACKUPTIME",
                                    "intraFieldOp": 0,
                                    "fieldValues": {
                                        "values": [
                                            "0",
                                            str(options["to_time"])
                                        ]
                                    }
                                },
                                {
                                    "field": "DATA_TYPE",
                                    "intraFieldOp": 0,
                                    "fieldValues": {
                                        "values": [
                                            "1"
                                        ]
                                    }
                                }
                            ]
                        }
                    }
                ],
                "emailFilter": [],
                "galaxyFilter": [
                    {
                        "appIdList": [
                            int(options["subclient_id"])
                        ]
                    }
                ],
                "cvSearchKeyword": {
                    "isExactWordsOptionSelected": False,
                    "keyword": str(options["attribute"]) + "*"
                }
            },
            "searchProcessingInfo": {
                "resultOffset": 0,
                "pageSize": 15,
                "queryParams": [
                    {
                        "param": "ENABLE_MIXEDVIEW",
                        "value": "true"
                    },
                    {
                        "param": "ENABLE_NAVIGATION",
                        "value": "on"
                    },
                    {
                        "param": "ENABLE_DEFAULTFACETS",
                        "value": "false"
                    },
                    {
                        "param": "RESPONSE_FIELD_LIST",
                        "value": "CONTENTID,CV_TURBO_GUID,PARENT_GUID,AFILEID,AFILEOFFSET,"
                                 "COMMCELLNO,MODIFIEDTIME,SIZEINKB,DATA_TYPE,AD_DISPLAYNAME,"
                                 "AD_ID,AD_OBJECT_TYPE,BACKUPTIME,AD_FLAGS,CISTATE,DATE_DELETED,"
                                 "AD_MAIL,AD_MAILNICKNAME,AD_PROXY_ADDRESSES,AD_BUSINESS_PHONES,"
                                 "AD_CITY,AD_COUNTRY,AD_DELETED_TIME,AD_POSTALCODE,AD_STATE,"
                                 "AD_STREET_ADDRESS,AD_LAST_DIR_SYNC_TIME,"
                                 "AD_COUNTRY_LETTER_CODE,AD_DIR_SYNC_ENABLED,AD_MKT_NOTIFY_MAILS,"
                                 "AD_TENANT_OBJECT_TYPE,AD_PREFER_LANG,AD_SEC_NOTIFY_MAILS,"
                                 "AD_SEC_NOTIFY_PHONES,AD_TECH_NOTIFY_MAILS,AD_TELEPHONE_NR,"
                                 "AD_CREATED_TIME,AD_DESCRIPTION,AD_GROUP_TYPES,AD_MAIL_ENABLED,"
                                 "AD_VISIBILITY,AD_SOURCE_TYPE,AD_AZURE_APP_ID,AD_HOME_PAGE_URL,"
                                 "AD_TAGS,AD_AZURE_APP_DISPLAY_NAME,AD_APP_OWNER_ORGID,"
                                 "AD_REPLY_URLS,AD_PUBLISHER_NAME,AD_SERVICE_PRINCIPAL_NAMES"
                    },
                    {
                        "param": "DO_NOT_AUDIT",
                        "value": "true"
                    },
                    {
                        "param": "COLLAPSE_FIELD",
                        "value": "AD_ID"
                    },
                    {
                        "param": "COLLAPSE_SORT",
                        "value": "BACKUPTIME DESC"
                    }
                ],
                "sortParams": [
                    {
                        "sortDirection": 0,
                        "sortField": "AD_DISPLAYNAME"
                    }
                ]
            },
            "facetRequests": {
                "facetRequest": [
                    {
                        "name": "AD_OBJECT_TYPE"
                    }
                ]
            }
        }

        return request_json

    def get_search_response(self, job_time, attribute):
        """
            Searches for jobs based on the specified parameters.

            This method performs a search operation for jobs using the
            given job time, display name and application ID.
            Args:
                job_time    (str)   The job ends time.
                attribute   (str)  Attribute to search for.
            Return:
                The response contains the search results.
            Raises:
                SDKException: If there is a bad request
                or no object found with the specified display name.
        """

        uri = self._services["DO_WEB_SEARCH"]
        options = {"to_time": job_time, "attribute": attribute}
        request_json = self.__prepare_search_json(options)
        flag, response = self._cvpysdk_object.make_request('POST', uri, request_json)
        if not flag:
            raise SDKException('Response', '101', self._update_response_(response.text))
        response = response.json()
        if response["proccessingInfo"]["totalHits"] == 0:
            raise SDKException('Backupset', '107', "no result found with specified attribute")
        return response

    def view_attributes_url_builder(self,
                                    job_time,
                                    display_name):
        """
        Builds a URL for viewing attributes based on the specified parameters.

        This method constructs a URL for viewing attributes of an
        object identified by the given job time and display name.
        Args:
            job_time    (str)   The job time.
            display_name    (str)   The display name of the object.
        Return:
            The URL for viewing attributes.
        """

        subclient_id = self.subclients.all_subclients['default']['id']

        try:
            search_response = self.get_search_response(job_time=job_time, attribute=display_name)
            afile_id = search_response["searchResult"]["resultItem"][0]["aFileId"]
            afile_offset = search_response["searchResult"]["resultItem"][0]["aFileOffset"]
            commcell_no = search_response["searchResult"]["resultItem"][0]["commcellNo"]

            op = "dGVtcC5qc29u"  # any filename.json for view properties call
            # encoding string to base64
            stub_info = str('2:' + str(commcell_no) + ':0:' +
                            str(afile_id) + ':' + str(afile_offset))
            stub_info = stub_info.encode("ascii")
            stub_info = base64.b64encode(stub_info)
            stub_info = stub_info.decode("ascii")

            url = self._services['VIEW_PROPERTIES'] % (str(self._agent_object.agent_id),
                                                       stub_info,
                                                       op,
                                                       str(subclient_id),
                                                       '1')
            return url
        except SDKException as e:
            raise SDKException('Backupset', '107',
                               f"No result found with specified attribute {e.exception_message}")

    def get_view_attribute_response(self,
                                    job_time,
                                    display_name):
        """
        Retrieves view attributes based on the specified parameters.

        This method retrieves the view attributes of an
        object identified by the given job time and display name.
        Args:
            job_time    (int)   The job time.
            display_name    (string)    The display name of the object.
        Return:
            (dict)  The JSON response contains the view attributes.
        Raises:
            SDKException: If there is an error while retrieving the view attributes.
        """
        try:
            url = self.view_attributes_url_builder(display_name=display_name,
                                                   job_time=job_time)

            flag, response = self._cvpysdk_object.make_request('GET', url)
            if not flag:
                raise SDKException('Response', '101', "Response was not success")
            if not response:
                raise SDKException('Response', '102', "Response received is empty")
            return response.json()
        except Exception as e:
            raise SDKException('Backupset', '107', f"No result found {e.exception_message}")



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# --------------------------------------------------------------------------

"""Main file for performing operations on content analyzers, and a single content analyzer client in the commcell.

`ContentAnalyzers`, and `ContentAnalyzer` are 2 classes defined in this file.

ContentAnalyzers:    Class for representing all the Content analyzers in the commcell.

ContentAnalyzer:     Class for representing a single content analyzer client in the commcell.


ContentAnalyzers:

    __init__(commcell_object)           --  initialise object of the ContentAnalyzers class

     _response_not_success()            --  parses through the exception response, and raises SDKException

    refresh()                           --  refresh the content analyzers associated with the commcell

    get()                               --  Returns an instance of ContentAnalyzer class for the given CA client name

    get_properties()                    --  Returns the properties for the given content analyzer client name

    _get_all_contentanalyzers()         --  Returns dict consisting all content analyzers associated with commcell

    _get_cloud_from_collections()       --  gets all the content analyzer details from collection response

    has_client()                        --  Checks whether given CA client exists in commcell or not


ContentAnalyzer:

    __init__()                          --  initialize an object of ContentAnalyzer Class with the given CACloud
                                                name and client id associated to the commcell

    refresh()                           --  refresh the properties of the CA client

    _get_cloud_properties()             --  Gets all the details of associated content analyzer client


ContentAnalyzer Attributes
-----------------

    **client_id**    --  returns the client id of the content analyzer client

    **cloud_url**    --  returns the url of the content analyzer

"""
from .exception import SDKException


class ContentAnalyzers(object):
    """Class for representing all the ContentAnalyzers in the commcell."""

    def __init__(self, commcell_object):
        """Initializes an instance of the ContentAnalyzers class.

            Args:
                commcell_object     (object)    --  instance of the commcell class

            Returns:
                object  -   instance of the ContentAnalyzers class

        """
        self._commcell_object = commcell_object
        self._update_response_ = commcell_object._update_response_
        self._cvpysdk_object = commcell_object._cvpysdk_object
        self._services = commcell_object._services
        self._content_analyzers = None
        self._api_get_content_analyzer_cloud = self._services['GET_CONTENT_ANALYZER_CLOUD']
        self.refresh()

    def _response_not_success(self, response):
        """Helper function to raise an exception when reponse status is not 200 (OK).

            Args:
                response    (object)    --  response class object,

                received upon running an API request, using the `requests` python package

        """
        raise SDKException('Response', '101', self._update_response_(response.text))

    def get_properties(self, caclient_name):
        """Returns a properties of the specified content analyzer client name.

            Args:
                caclient_name (str)  --  name of the content analyzer client

            Returns:
                dict -  properties for the given content analyzer client name


        """
        return self._content_analyzers[caclient_name.lower()]

    def _get_all_content_analyzers(self):
        """Gets the list of all content analyzers associated with this commcell.

            Returns:
                dict    -   dictionary consisting of dictionaries, where each dictionary stores the
                                details of a single content analyzer

                    {
                        "contentAnalyzerList": [
                                {
                                    "caUrl": "",
                                     "clientName": "",
                                     "clientId": 0
                                },
                                {
                                      "caUrl": "",
                                      "clientName": "",
                                      "clientId": 0
                                }
                        ]
                    }

            Raises:
                SDKException:
                    if response is empty

                    if response is not success

        """
        flag, response = self._commcell_object._cvpysdk_object.make_request(
            'GET', self._api_get_content_analyzer_cloud
        )
        if flag:
            if response.json() and 'contentAnalyzerList' in response.json():
                return self._get_cloud_from_collections(response.json())
            raise SDKException('ContentAnalyzer', '103')
        self._response_not_success(response)

    @staticmethod
    def _get_cloud_from_collections(collections):
        """Extracts all the content analyzers, and their details from the list of collections given,
            and returns the dictionary of all content analyzers.

            Args:
                collections     (list)  --  list of all collections

            Returns:
                dict    -   dictionary consisting of dictionaries, where each dictionary stores the
                                details of a single content analyzer

        """
        _cacloud = {}
        for cacloud in collections['contentAnalyzerList']:
            cacloud_dict = {}
            cacloud_dict['caUrl'] = cacloud.get('caUrl', "")
            cacloud_dict['clientName'] = cacloud.get('clientName', "")
            cacloud_dict['clientId'] = cacloud.get('clientId', 0)
            _cacloud[cacloud['clientName'].lower()] = cacloud_dict
        return _cacloud

    def refresh(self):
        """Refresh the content analyzers associated with the commcell."""
        self._content_analyzers = self._get_all_content_analyzers()

    def get(self, client_name):
        """Returns a ContentAnalyzer object for the given CA client name.

            Args:
                client_name (str)    --  name of the Content analyzer client

            Returns:

                obj                 -- Object of ContentAnalyzer class

            Raises:
                SDKException:
                    if response is empty

                    if response is not success

                    if cacloud_name is not of type string


        """
        if not isinstance(client_name, str):
            raise SDKException('ContentAnalyzer', '101')

        if self.has_client(client_name):
            return ContentAnalyzer(self._commcell_object, client_name)
        raise SDKException('ContentAnalyzer', '102', "Unable to get ContentAnalyzer class object")

    def has_client(self, client_name):
        """Checks if a content analyzer client exists in the commcell with the input name.

            Args:
                client_name (str)    --  name of the content analyzer client

            Returns:
                bool - boolean output whether the CA client exists in the commcell or not

            Raises:
                SDKException:
                    if type of the CA client name argument is not string

        """
        if not isinstance(client_name, str):
            raise SDKException('ContentAnalyzer', '101')

        return self._content_analyzers and client_name.lower() in map(str.lower, self._content_analyzers)


class ContentAnalyzer(object):
    """Class for performing operations on a single content analyzer client"""

    def __init__(self, commcell_object, client_name):
        """Initialize an object of the ContentAnalyzer class.

            Args:
                commcell_object     (object)    --  instance of the commcell class

                client_name     (str)           --  name of the content analyzer client

            Returns:
                object  -   instance of the ContentAnalyzer class
        """
        self._commcell_object = commcell_object
        self._client_name = client_name
        self._cloud_url = None
        self._client_id = self._commcell_object.clients.get(client_name).client_id
        self.refresh()

    def _get_cloud_properties(self):
        """ Get properties for all content analyzers client in the commcell
                Args:

                    None

                Returns:

                    None

        """
        content_analyzers_dict = self._commcell_object.content_analyzers.get_properties(self._client_name)
        self._cloud_url = content_analyzers_dict['caUrl']
        return content_analyzers_dict

    @property
    def client_id(self):
        """Returns the value of the Content analyzer client id attribute."""
        return int(self._client_id)

    @property
    def cloud_url(self):
        """Returns the value of the Content analyzer client url attribute."""
        return self._cloud_url

    def refresh(self):
        """Refresh the content analyzer details associated with this commcell"""
        self._get_cloud_properties()



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# --------------------------------------------------------------------------
"""
File for Ad agent sublcient replated operation

Class:
    ADSubclient :

        _get_subclient_properties()    --    Method to get subclinet properites

        _get_subclient_properties_json --    Method to generate subclinet properties in json format

        content                        --    Properties of AD objects in subclient

        compare_id                     --    Method returns AD compare id

        trigger_compare_job            --    Method to trigger AD compare job

        checkcompare_result_generated  --    Method to check AD compare report generated

        generate_compare_report        --    Method to generate AD compare report

        restore_job                    --    Method to do AD point in time restore
ADSubclient:
    content() -- method to get AD agent subclient content

Function:
"""

from __future__ import unicode_literals

from ..exception import SDKException
from ..subclient import Subclient
import time

class ADSubclient(Subclient):
    """Class for AD agent related subclient """

    def _get_subclient_properties(self):
        """Gets the subclient  related properties of File System subclient.
        """
        super()._get_subclient_properties()
        if 'impersonateUser' in self._subclient_properties:
            self._impersonateUser = self._subclient_properties['impersonateUser']
        if 'content' in self._subclient_properties:
            self._content = self._subclient_properties['content']

    def _get_subclient_properties_json(self):
        """get the all subclient related properties of this subclient.
           Returns:
                dict - all subclient properties put inside a dict
        """
        subclient_json = {
            "subClientProperties":
                {
                    "impersonateUser": self._impersonateUser,
                    "proxyClient": self._proxyClient,
                    "subClientEntity": self._subClientEntity,
                    "content": self._content,
                    "commonProperties": self._commonProperties,
                    "contentOperationType": 1
                }
        }
        return subclient_json

    @property
    def content(self):
        """ Get AD agent subclient content"""
        contents = []
        if "content" in self._subclient_properties:
            subclient_content = self._subclient_properties['content']
        else:
            subclient_content = []
        if len(subclient_content) > 0:
            for entry in subclient_content:
                if len(entry['path'].split(',')) > 1:
                    contents.append(",".join(entry['path'].split(',')[1:]))
                else:
                    raise SDKException('Subclient', "101",
                                       "subclient content is not valid")
        else:
            raise SDKException('Subclient', '101',
                               "subclient content return empty result")

        return contents

    def cv_contents(self, contents, entrypoint=None):
        """Commvault subclient content convert to AD format
            Args:
                content    (list)    subclient content
                entrypoint    (string)    ad object entry point
            Return:
                (list)    ad format content
            Exception:
                None
        """
        content_ad = []
        for content in contents:
            contententry = ",".join(list(reversed(content.split(","))))
            if entrypoint is not None:
                shortdn = contententry.split(",{0}".format(entrypoint))[0]
            else:
                shortdn = None
            content_ad.append((contententry, shortdn))
        return content_ad

    def compare_id(self,left_set_time,right_set_time,
                   source_item,comparison_name):
        """
        Generate Commvault AD Compare Comparison id
        Args:
             left_set_time  (int)      End Time of the first backup for comparison
             right_set_time (int)      End Time of the second backup for comparison
             source_item    (string)   sourceItem for comparison
             comparison_name(string)   name of the comparison
        Return:
            (int) Comparison ID
        Exception:
                Response was not a success
        """
        client_id=int(self._client_object.client_id)
        subclient_id=int(self.subclient_id)

        payload = {
            "adCompareOptions": {"adCompareType": 0, "comparisonName": comparison_name,
                                 "subClientId": (subclient_id),
                                 "appTypeId": 41, "clientId": (client_id),
                                 "nodeClientId": (client_id),
                                 "leftSetTime": (left_set_time),
                                 "rightSetTime": (right_set_time), "status": 0,
                                 "adComparisonJobType": 0,
                                 "selectedItems": {"sourceItem": [source_item]},
                                 "includeUnchangedItems": False,
                                 "excludeFrequentlyChanged": True}}
        generate_comparison_id = self._services['ADCOMPAREID'] % (subclient_id)

        flag, response = self._cvpysdk_object.make_request(
           method= 'POST',url= generate_comparison_id,payload= payload
        )

        if flag:
            response_dict=response.json()
            comp_id = response_dict['adCompareDetails'][0]['comparisonId']


        else:

            raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))

        return comp_id

    def trigger_compare_job(self,left_set_time,
                            right_set_time,
                            display_name,client_name,
                            comp_id,source_item,comparison_name):
        """
        Triggers Commvault AD compare job
        Args:
              left_set_time (int) End Time of the first backup for comparison
              right_set_time(int) End Time of the second backup for comparison
              display_name(str) displayName of the client
              client_name(str) name of the client
              comp_id(int) the comparison id for AD compare
              source_item(str)  sourceItem for comparison
              comparison_name(str) name of the comparison
        Return:
            None
        Exception:
                Response was not a success
        """
        client_id = int(self._client_object.client_id)
        backupset_id=int(self._backupset_object._get_backupset_id())

        subclient_id = int(self.subclient_id)

        trigger_compare_job=self._services['CREATE_TASK']
        request_time=int(time.time())
        payload = {"taskInfo": {"associations": [
            {"subclientId": (subclient_id), "displayName":
                display_name, "clientId": (client_id),
             "applicationId": 41, "clientName":
                 client_name, "backupsetId": (backupset_id),
             "_type_": "SUBCLIENT_ENTITY"}],
            "task": {"taskType": 1}, "subTasks": [
            {"subTask": {"subTaskType": 1, "operationType": 4025},
             "options": {"backupOpts": {},
                         "commonOpts": {"notifyUserOnJobCompletion": False},
                         "adComparisonOption": {"adCompareType": 0,
                                                "comparisonName": comparison_name,
                                                "subClientId": (subclient_id),
                                                "appTypeId": 41,
                                                "clientId": (client_id),
                                                "nodeClientId": (client_id),
                                                "leftSetTime": left_set_time,
                                                "rightSetTime": right_set_time,
                                                "status": 0,
                                                "adComparisonJobType": 0,
                                                "selectedItems": {"sourceItem": [source_item]},
                                                "includeUnchangedItems": False,
                                                "excludeFrequentlyChanged": True,
                                                "comparisonId": (comp_id),
                                                "requestTime":request_time ,
                            "selectionHash": "D51A0DDBBFF582EEFA23A750DB90F1A46F6E90CE"}}}]}}


        flag, response = self._cvpysdk_object.make_request(method='POST',
                                                                   url=trigger_compare_job,
                                                                     payload=payload)

        if not flag:
            raise SDKException('Response', '101',
                               self._commcell_object._update_response_(response.text))

    def checkcompare_result_generated(self,comp_id):
        """
        Function to check AD comparison result generated and returns comparison cache path
        Args:
             comp_id(int): comparison id of AD compare operation
        Return:
            (string) comparisonCachePath
        Exception:
                Response was not a success
        """
        compare_result = self._services['ADCOMPARESTATUSCHECK'] % (comp_id)

        while True:

            flag, response = self._cvpysdk_object.make_request(method='GET',
                                                                       url=compare_result)

            if flag:

                response_dict = response.json()
                error_message = response_dict['adCompareDetails'][0]['errorMessage']
                status = response_dict['adCompareDetails'][0]['status']
                comparison_cache_path = response_dict['adCompareDetails'][0]['sqlLiteDBCachePath']

                if (error_message == "" and status == 0 and comparison_cache_path != ""):
                    return comparison_cache_path
                if error_message != "":

                    raise SDKException('Response', '101',
                                       self._commcell_objectj._update_response_(response.text))

                time.sleep(30)
            else:
                raise SDKException('Response', '101',
                                   self._commcell_object._update_response_(response.text))

    def generate_compare_report(self,comp_id,comparison_cache_path,op_type=2):
        """
        Return AD Compare report
            Args:
                comp_id (int) the comparison id generate
                comparison_cache_path (str) local directory where we write the comparison file
            Return:
                (json) AD Compare Report
            Exception:
                    Response was not a success
        """
        client_id = int(self._client_object.client_id)
        subclient_id = int(self.subclient_id)
        ad_comp_url = self._services['ADCOMPAREVIEWRESULTS']
        payload = {"isEnterprise": True, "discoverySentTypes": [24],
                     "subclientDetails": {"subclientId": (subclient_id), "appTypeId": 41,
                                          "clientId": (client_id)},
                     "adComparisonBrowseReq": {"pageNumber": 0, "comparisonId": comp_id,
                                               "opType": op_type, "browseItemId": 0,
                                               "pageSize": 15, "comparisonCachePath":
                                                   comparison_cache_path,
                                               "searches":[{"masks":["2","1","0"],"searchType":2}]}}


        flag, response = self._cvpysdk_object.make_request(method='POST', url=ad_comp_url,
                                                                       payload=payload)

        if not flag:
            raise SDKException('Response', '101',
                               self._commcell_object._update_response_(response.text))

        return response.json()

    def restore_job(self,display_name,client_name,subclient_name,to_time,restore_path):
        """
        Triggers AD restore job and waits for its completion
        Args:
              display_name(str) displayName of the client
              client_name(str) name of the client
              subclient_name(str) name of the subclient
              to_time(int) time from where to restore
              restore_path(str)  restore path
        Return:
             None
        Exception:
             Response was not a success
        """
        client_id = int(self._client_object.client_id)
        subclient_id = int(self.subclient_id)
        backupset_id = int(self._backupset_object._get_backupset_id())
        client_guid=self._client_object.client_guid
        commcell_id=int(self._commcell_object.commcell_id)
        instance_id=int(self._instance_object.instance_id)

        payload={"taskInfo":{"associations":[{"subclientId":subclient_id,
                                              "displayName":display_name,
                                              "clientId":client_id,
                                              "entityInfo":{"companyId":0,"companyName":""},
                                              "instanceName":"DefaultInstanceName",
                                              "applicationId":41,
                                              "clientName":client_name,
                                              "backupsetId":backupset_id,
                                              "instanceId":instance_id,
                                              "clientGUID":client_guid,
                                              "subclientName":subclient_name,
                                              "backupsetName":"defaultBackupSet","_type_":7}],
                                              "task":{"taskType":1,"initiatedFrom":1},
                                              "subTasks":[{"subTask":
                                              {"subTaskType":3,"operationType":1001},
                                              "options":{"commonOpts":{
                                              "notifyUserOnJobCompletion":False,
                                              "jobMetadata":[{"selectedItems":
                                              [{"itemName":"","itemType":""}],
                                              "jobOptionItems":[]}]},
                                              "restoreOptions":{"browseOption":
                                              {"commCellId":commcell_id,
                                              "noImage":False,"useExactIndex":False,
                                              "listMedia":False,"backupset":
                                              {"backupsetId":backupset_id,
                                              "clientId":client_id},
                                              "timeRange":{"toTime":to_time},
                                              "mediaOption":{"copyPrecedence":
                                              {"copyPrecedenceApplicable":False}}},
                                              "destination":{"inPlace":True,"destClient":
                                              {"clientId":client_id,"clientName":client_name}},
                                              "fileOption":{"sourceItem":[restore_path]},
                                              "activeDirectoryRstOption":{},
                                              "commonOptions":{"detectRegularExpression":True,
                                              "preserveLevel":1,"restoreACLs":True}}}}]}}

        trigger_response=self._services['CREATE_TASK']
        flag, response =self._cvpysdk_object.make_request(method='POST', url=trigger_response,
                                                                   payload=payload)
        if not flag:
            raise SDKException('Response', '101',
                               self._commcell_object._update_response_(response.text))
        if not response:
            raise SDKException('Response', '101',
                               self._commcell_object._update_response_(response.text))
        response = response.json()
        if "errorCode" in response:
            raise SDKException('Response', '101',
                               self._commcell_object._update_response_(response.text))
        # get restore job and wait for its completion
        job_id = response["jobIds"][0]
        job = self._commcell_object.job_controller.get(job_id)
        job.wait_for_completion()


# -*- coding: utf-8 -*-

# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# --------------------------------------------------------------------------

"""
Main file for performing CommServe Recovery operations.

CommServeRecovery class is defined in this file.

CommServeRecovery:      Class for Commserve Recovery of a CS, and to perform operations related to Commserve Recovery


CommServeRecovery:
    __init__()                          --   initialize object of CommServeRecovery with the required CS GUID

    _get_backupsets()                   --   method to get the latest 5 backupsets that are shown on the cloud command

    _get_active_recovery_requests()     --   method to get all the recovery requests submitted for the CS

    _quota_details()                    --   returns the current recovery license status of the Commserv

    _create_cs_recovery_request()       --   returns the details of the given request ID

"""
from __future__ import absolute_import
from __future__ import unicode_literals

from json import JSONDecodeError
from datetime import datetime
from cvpysdk.exception import SDKException
from cvpysdk.commcell import Commcell


class CommServeRecovery:
    """Class to perform operations related to Commserve Recovery"""

    def __init__(self, commcell_object: Commcell, cs_guid: str):
        """Initialize the instance of the CommServeRecovery class.

            Args:
                commcell_object   (commcell object)    --  instance of the Commcell class

                cs_guid           (str)                --  CS GUID

        """
        self._commcell_object = commcell_object
        self._cvpysdk_object = commcell_object._cvpysdk_object
        self._services = commcell_object._services
        self.cs_guid = cs_guid
        self._CS_RECOVERY_API = self._services['COMMSERVE_RECOVERY']
        self._CS_RECOVERY_LICENSE_API = self._services['GET_COMMSERVE_RECOVERY_LICENSE_DETAILS'] % self.cs_guid
        self._CS_RECOVERY_RETENTION_API = self._services['GET_COMMSERVE_RECOVERY_RETENTION_DETAILS'] % self.cs_guid
        self._BACKUPSET_API = self._services['GET_BACKUPSET_INFO'] % self.cs_guid
        self._cleanup_lock_time = None
        self._is_licensed_commcell = self._quota_details().get('is_licensed')

        self._manual_retention_details()

    def _get_backupsets(self):
        """Returns details of uploaded backupsets"""
        flag, response = self._cvpysdk_object.make_request('GET', self._BACKUPSET_API)

        if flag:
            try:
                companies = response.json().get('companies')
                if not companies:
                    raise SDKException('CommserveRecovery', '101')
                return {
                    backupset_details["set_name"]: {
                        "set_id": backupset_details["set_id"],
                        "size": sum(int(files_details['size']) for files_details in backupset_details['files']),
                        "backup_time": datetime.strptime(backupset_details["time_modified"], "%Y-%m-%dT%H:%M:%SZ").timestamp(),
                        "manually_retained": bool(self._cleanup_lock_time),
                        "retained_until": self._cleanup_lock_time if self._cleanup_lock_time else None
                    }
                    for backupset_details in companies[0].get('commcells')[0].get('sets')
                }


            except (JSONDecodeError, KeyError):
                raise SDKException('Responcsse', '102', 'Job id not found in response')
        else:
            raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))

    def _create_cs_recovery_request(self, set_name: str):
        """submits a new commserve recovery request and returns the request id to track"""
        try:
            set_id, set_size = self.backupsets[set_name]['set_id'], self.backupsets[set_name]['size']
        except KeyError:
            raise SDKException("CommserveRecovery", "104")
        payload = {
            "commcellGUID": self.cs_guid,
            "setId": set_id,
            "setName": set_name,
            "setSize": set_size
        }
        flag, response = self._cvpysdk_object.make_request('POST', self._CS_RECOVERY_API, payload)

        if flag:
            try:
                if not response.json()['success']:
                    raise SDKException('Response', '101', 'request creation not successful')
                return response.json()['requestId']
            except (JSONDecodeError, KeyError):
                raise SDKException('Responcsse', '102', 'Job id not found in response')
        else:
            raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))

    def _extend_recovery_request(self, request_id: int):
        """Returns True if the request is submitted successfully, otherwise, False"""
        payload = {
            "csGuid": self.cs_guid,
            "requestId": request_id,
            "operation": 2
        }
        flag, response = self._cvpysdk_object.make_request('PUT', self._CS_RECOVERY_API, payload)

        if flag:
            if response.json()['errorCode'] == 0:
                return True
            else:
                raise SDKException('Response', '102', 'request did not submit successfully')
        else:
            raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))

    def _get_active_recovery_requests(self):

        url = f'{self._CS_RECOVERY_API}?csGuid={self.cs_guid}&showOnlyActiveRequests=true'

        states = {
            1: 'SUBMITTED',
            2: 'CREATING_VM',
            3: 'VM_CREATED',
            4: 'STAGING_CS',
            5: 'CS_STAGED',
            6: 'FINISHED',
            7: 'FAILED',
            8: 'KILLED'
        }

        flag, response = self._cvpysdk_object.make_request('GET', url)

        if flag:
            response = response.json()
            if response.get("errorCode") == 6:
                raise SDKException('CommserveRecovery', '101')

            map_vm_info = lambda vm_info: {
                                    "commandcenter_url": f"https://{vm_info['ipAddress']}/commandcenter",
                                    "vm_expiration_time": vm_info['vmExpirationTime'],
                                    "username": vm_info['credentials']['sUsername'],
                                    'password': vm_info['credentials']['sPassword']
                                }
            return {
                        request["id"]: {
                            "backupset": request.get("setName"),
                            "requestor": request["requestor"]["fullName"],
                            "version": request["servicePack"],
                            "start_time": request["createdTime"],
                            "end_time": request["vmInfo"]['vmExpirationTime'],
                            "status": states[request["status"]],
                            "vmInfo": map_vm_info(request['vmInfo']) if request['status'] == 5 else {}
                        }
                        for request in response.get('requests', [])
                    }

        else:
            raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))

    def _get_vm_details(self, requestid: int):
        """Returns VM access details"""
        try:
            return self.active_recovery_requests[requestid]['vmInfo']
        except KeyError:
            raise SDKException('CommserveRecovery', '103')

    def _quota_details(self):
        """
        Returns CS Recovery license quota details in a dictionary
        """
        flag, response = self._cvpysdk_object.make_request('GET', self._CS_RECOVERY_LICENSE_API)

        if flag:
            response = response.json()
            return {
                "is_licensed": response['license'],
                "start_date": datetime.strptime(response['quotaStartDate'], "%Y-%m-%dT%H:%M:%SZ").timestamp(),
                "end_date": datetime.strptime(response['quotaEndDate'], "%Y-%m-%dT%H:%M:%SZ").timestamp(),
                'used_recoveries_count': response['recoveriesCount'],
                'total_recoveries_allocated': response['maxRecoveries']
            }
        else:
            raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))

    def _manual_retention_details(self):
        """Returns details of used vs allocated quota of manual retains in a dict"""
        flag, response = self._cvpysdk_object.make_request('GET', self._CS_RECOVERY_RETENTION_API)
        if flag:
            response = response.json()
            self._cleanup_lock_time = response['cleanup_lock_time']
            return {
                "start_date": datetime.strptime(response['quota_start_date'], "%Y-%m-%dT%H:%M:%SZ").timestamp(),
                "end_date": datetime.strptime(response['quota_end_date'], "%Y-%m-%dT%H:%M:%SZ").timestamp(),
                'consumed_retains': response['consumed_retains'],
                'max_retains_allocated': response['max_retains']
            }
        else:
            raise SDKException('Response', '101', self._commcell_object._update_response_(response.text))

    @property
    def is_licensed_commcell(self):
        return self._is_licensed_commcell

    @property
    def backupsets(self):
        return self._get_backupsets()

    @property
    def active_recovery_requests(self):
        return self._get_active_recovery_requests()

    @property
    def recovery_license_details(self):
        """
        Returns a dict containing start date, expiry date of license, number of consumed recovery requests and max requests count.
        Ex:
        {
                "is_licensed": 1,
                "start_date": 1711460013.0,
                "end_date": 1711460013.0,
                'used_recoveries_count': 23,
                'total_recoveries_allocated': 40
            }
        """
        return self._quota_details()

    @property
    def manual_retention_details(self):
        """
        Returns a dict containing start date, expiry date of license, number of consumed retains and max retains allocated.
        Ex:
        {
            "start_date": 1711460013,
            "end_date": 1711544749,
            'consumed_retains': 4,
            'max_retains_allocated': 10
        }
        """
        return self._manual_retention_details()

    def start_recovery(self, backupset_name: str):
        """
        Submits commserve recovery request for the given backupset
        Args:
            backupset_name (str) : name of the backup set to recover.
                                    Ex: SET_45
        Returns:
            Request id of the submitted request as an integer
        """
        return self._create_cs_recovery_request(backupset_name)

    def extend_reservation(self, request_id: int):
        """
        Extends the expiry time of a VM created for the given request id.
        Args:
            request_id (int) : commserve recovery request id
        Returns:
            True is the request is submitted successfully, otherwise, False
        """
        return self._extend_recovery_request(request_id)

    def get_vm_details(self, request_id: int):
        """
        Returns VM details for the given recovery request in a dict format.
        Ex:
        {
            "commandcenter_url": "https://20.235.143.244/commandcenter",
            "vm_expiration_time": 1712311000,
            "username": "recoverymanager",
            "password": "<REDACTED>",
        }
        """
        return self._get_vm_details(request_id)
